@@ -1,5 +1,22 @@
 package Pg::Explain::From;
+
+# UTF8 boilerplace, per http://stackoverflow.com/questions/6162484/why-does-modern-perl-avoid-utf-8-by-default/
+use v5.14;
 use strict;
+use warnings;
+use warnings qw( FATAL utf8 );
+use utf8;
+use open qw( :std :utf8 );
+use Unicode::Normalize qw( NFC );
+use Unicode::Collate;
+use Encode qw( decode );
+
+if ( grep /\P{ASCII}/ => @ARGV ) {
+    @ARGV = map { decode( 'UTF-8', $_ ) } @ARGV;
+}
+
+# UTF8 boilerplace, per http://stackoverflow.com/questions/6162484/why-does-modern-perl-avoid-utf-8-by-default/
+
 use Pg::Explain::Node;
 use Carp;
 
@@ -9,11 +26,11 @@ Pg::Explain::From - Base class for parsers of non-text explain formats.
 
 =head1 VERSION
 
-Version 0.96
+Version 0.97
 
 =cut
 
-our $VERSION = '0.96';
+our $VERSION = '0.97';
 
 =head1 SYNOPSIS
 
@@ -111,7 +128,8 @@ sub make_node_from {
         'actual_loops'           => $struct->{ 'Actual Loops' },
     );
     $new_node->explain( $self->explain );
-    if ( $struct->{ 'Actual Loops' } == 0 ) {
+
+    if ( !$struct->{ 'Actual Loops' } ) {
         $new_node->never_executed( 1 );
     }
 
